//
//  Koupon.h
//  Koupon
//
//  Created by Brian Reinhart on 9/24/13.
//  Copyright (c) 2013 Brian Reinhart. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "KMRequest.h"
#import "KMLocation.h"
#import <stdlib.h>

/** This class contains all of the API wrappers for Koupon Media APIs.
 
 The init function requires a plist file named KouponConfiguration.plist.  The plist file should contain the authKey and authSecret strings provided by Koupon Media.
 
 After initialization, you must first obtain a consumerId for the logged in user.
 
    Koupon* kouponSDK = [[Koupon alloc] init];
 
    //Post data to send to createConsumerIdentity API
    NSMutableDictionary* data = [[NSMutableDictionary alloc] init];
    [data setObject:@"blahblah@email.com" forKey:@"email"];
    
    //Get the KMRequest object with the signed URL.
    KMRequest* request = [kouponSDK createConsumerIdentity:data withOptionalQueryData:nil];
 
    //You can either roll your own connection management, or execute the request directly from the KMRequest Object
    NSMutableDictionary* results = [request sendRequest];
 
    //Now set the consumerId for all future calls.
    [kouponSDK setConsumerId:[results valueForKey:@"consumer_identity"]];
 
 */

@interface Koupon : NSObject {
    @private
    NSString *channel;
    NSString *dpCode;
    NSString *version;
    NSDate *sessionStartTime;
    NSTimer *updateTimer;
    NSDictionary *sdkConfig;
    BOOL didReceiveRemoteNotificationWhileClosed;
}

/**---------------------------------------------------------------------------------------
 * @name Properties
 *  ---------------------------------------------------------------------------------------
 */

/** The authKey provided by Koupon Media
 */
@property NSString *authKey;
/** The authSecret used for signing requests
*/
@property NSString *authSecret;
/** The consumer Identity returned back from createConsumerIdentity
 
/** The root domain used.  Should be http://consumer.kouponmedia.com for all production requests.
*/
@property (nonatomic, strong) NSString *domain;
@property (nonatomic, strong) NSString *appEventDomain;
@property (nonatomic, strong) NSString *dpCode;
@property (nonatomic, strong) NSString *version;
@property (nonatomic, strong) NSString *channel;
@property (nonatomic) BOOL debugLogging;

+(NSString*) consumerId;
+(void) setConsumerId:(NSString *) value;
+ (BOOL) automaticallyReportAppEvents;
+ (void) setAutomaticallyReportAppEvents: (BOOL) value;

#pragma mark - Class Initialization

/**---------------------------------------------------------------------------------------
 * @name Initialization Methods
 *  ---------------------------------------------------------------------------------------
 */

/** Initialize the Koupon SDK with the KouponConfiguration.plist file.
 
 Requires that a KouponConfiguration.plist file exists and contains the following dictionary values.
 
 - (NSString*) authkey
 - (NSString*) authSecret
 @return Koupon
 */
-(id) init;

/** Initialize the Koupon SDK with the provided authKey and authSecret.
 
 @param aKey The authKey provided by Koupon Media
 @param aSecret the authSecret provided by koupon Media
 @return Koupon
 */
-(id) initWithAuthKey: (NSString*) aKey AuthSecret: (NSString*) aSecret Domain: (NSString *) aDomain Version: (NSString *) aVersion DPCode: (NSString *) aDPCode;


#pragma mark - Consumer Identity And Information
/**---------------------------------------------------------------------------------------
 * @name Koupon Media API wrapper methods
 *  ---------------------------------------------------------------------------------------
 */

/** Create a Consumer Identity required for all further API calls.  
 
 consumerData must contain one of the following identifying criteria.
 
 - mobile   //mobile phone number of consumer
 - email    //email address of the consumer
 - cid      //client-supplied internal CRM id of the consumer
 - identity //Legacy identity id of the consumer
 - udid     //unique id of the consumer's device
 
 You can provide optional data as well, such as:
 
 - vendorid //Apple's identifierForVender
 - dob // YYYY-MM-DD
 - locHistory //Array of location updates for use with smart offers.
 
 Request:
 
     {
         "cid": String,
         "email": String,
         "identity": String,
         "mobile": String,
         "udid":  String,
         "vendorid":  String,
         "dob": String,
         "locHistory" : [
             {  
                 isotime:  String,
                 lat:  String,
                 lon:  String  
             },  ...
         ]
     }
 
 Response:
 
     {
         “code”: Number,
         “message”: String,
         “version”: String,
         "consumer_identity": String,
         "sdkConfig" : {
             "locCapture" : {
                 "enable" : Boolean,
                 "bufferSize" : Number
             }
         }
     }
 
 
 @param consumerData A dictionary containing one or more of the unique identifying criteria for a consumer.
 @param optionalData An optional dictionary containing additional query parameters
 */
-(void) createConsumerIdentity: (NSMutableDictionary*) consumerData withOptionalQueryData: (NSMutableDictionary*) optionalData;

/** Update Consumer Information for the current consumer.
 
 Update Consumer Data.
 
 @see createConsumerIdentity
 
 @param consumerData A dictionary containing one or more of the unique identifying criteria for a consumer.
 @param optionalData An optional dictionary containing additional query parameters
 @return KMRequest
 */
-(KMRequest*) updateConsumerData: (NSMutableDictionary*) consumerData withOptionalQueryData: (NSMutableDictionary*) optionalData;

/** Get All Consumer Information From The Server.
 
 Retrieves all of the consumer identity information that koupon media has for the current consumer.

 @param optionalData An optional dictionary containing additional query parameters
 @return KMRequest
 */
-(KMRequest*) getConsumerData: (NSMutableDictionary*) optionalData;

#pragma mark - Getting Consumer Offers

/** Get all of the available offers for the current consumer.
 
 Optional Parameters can filter the results.  All Date values must follow iso8601 standards.  Available parameters are:
 
 - store_id //number (merchant id)
 - geo_location //string
 - channel  //string
 - from_date //iso8601 date
 - to_date //iso8601 date
 - digital_property_code //string
 - distribution_code //string
 
 @param offerFilterDictionary A dictionary containing containing any of the optional filter parameters.
 @param optionalData An optional dictionary containing additional query parameters
 @return KMRequest
 */
-(KMRequest*) getConsumerOfferList: (NSMutableDictionary*) offerFilterDictionary withOptionalQueryData: (NSMutableDictionary*) optionalData;

/** Get a specific offer for the current consumer.
 
 @param offerId The unique offer Id to retreive.
 @param optionalData An optional dictionary containing additional query parameters
 @return KMRequest
 @see getConsumerOfferList
 */
-(KMRequest*) getConsumerOffer: (NSString*) offerId withOptionalData: (NSMutableDictionary*) optionalData;

#pragma mark - Updating Consumer Offers

/** Set the Consumer State for the given offer.  The <offer_id_list> parameter can consist of one <offer_id> or multiple comma-separated <offer_id>s (e.g. “1281,1282,1283”).
 
 The State should be one of the following:
 
 - Access
 - Viewed
 - Done
 - Presented
 
 If the state is set to "Presented", the response should contain actual barcode information and any redeem time information. This method will also be a part of the overall redemption process. Then the user will go through the process of redeeming that specific coupon.
 
 Request:
 
     {
         “state”: String
     }
 Response:
 
     {
         “code”: Number,
         “message”: String,
         “version”: String
         “state”: String,
         “kmstateresp” : [            // ONLY for state of "Presented"
             {   
                 “offer”: “<consumer_id>/offers/<offer_id>”,
                 “Timer”: Number,
                 “BarcodeString”: String,
                 “BarcodeType”: String,
                 “BarcodeSymbology”: String,
                 “BarcodeImage”: URLString,
                 “redemptionAssets”: [                // ONLY for version 2.1+
                     { 
                         “BarcodeString”: String,
                         “BarcodeType”: String,
                         “BarcodeSymbology”: String,
                         “BarcodeImage”:” URLString  
                     },  ...
                 ]   
             },  ...
         ]
     }
 
 @param aState The state.
 @param offerList The Offer Id or list of Offer Ids.
 @param optionalData An optional dictionary containing additional query parameters
 @return KMRequest
 */
-(KMRequest*) updateConsumerState: (NSString*) aState forOffer: (NSString*) offerList
                 withOptionalData: (NSMutableDictionary*) optionalData;


/** Add a single offer to a user so that the offer is now available for use in that consumer_id’s list of offers
 
Server removes the existing offer in the saved list and saves the supplied offer to the consumer’s list of saved offers. Server may check the validity of the offers at the time of saving. The overall list details such as the time and version are updated. 
 
 @param anOffer The offer Id
 @param optionalData An optional dictionary containing additional query parameters
 @return KMRequest
 @see addMultipleOffersToConsumer:withOptionalData:
 */

-(KMRequest*) addOfferToConsumer: (NSString*) anOffer withOptionalData: (NSMutableDictionary*) optionalData;

/** Adds multiple offers to a specific user so that the offer is now available for use in that consumer_id’s list of offers
 
 Server removes the existing offer in the saved list and saves the supplied offer to the consumer’s list of saved offers. Server may check the validity of the offers at the time of saving. The overall list details such as the time and version are updated.
 
 @param offerList The list of offers
 @param optionalData An optional dictionary containing additional query parameters
 @return KMRequest
 @see addOfferToConsumer:withOptionalData:
 */

-(KMRequest*) addMultipleOffersToConsumer: (NSMutableArray*) offerList withOptionalData: (NSMutableDictionary*) optionalData;

/** Registers your deviceToken with the Koupon Media servers for use with Push Notifications

 You should call this mehtod after receiving the deviceToken inside of application:didRegisterForRemoteNotificationsWithDeviceToken:
 The below code assumes you have an instance of Koupon named kouponSDK.
 
    - (void) application:(UIApplication*) application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
    {
        [kouponSDK registerDeviceToken:deviceToken];
    }
 
 @param NSData The deviceToken returned from Apple
 @param NSMutableDictionary An optional dictionary containing additional query parameters
 @return KMRequest
 @see reportAppEvent:withData:
 @warning Deprecated.  Use reportAppEvent:withData:
 */

-(KMRequest*) registerDeviceToken: (NSData*) deviceToken withOptionalData: (NSMutableDictionary*) optionalData __deprecated;

/** Generates and downloads a passbook version of the offer
 
 This API should be called if a user wants to send an offer to their passbook
 
 A valid response from this request will be in the form of a .pkpass file.  You can load the pkpass into a passkitview.
 
 @param offerId The offerId to convert to passbook
 @param optionalData An optional dictionary containing additional query parameters
 @return KMRequest
 */

-(KMRequest*) generatePassbookOffer: (NSString*) offerId withOptionalData: (NSMutableDictionary*) optionalData;

/** Reports a geofence enter or exit event to Koupon Media.
 
 The API will return back a list of Offers for that area.  @see getConsumerOffer for return data format.
 
 @param fenceEvent The geofence event - either "enter" or "leave"
 @param fenceName The name of the geofence - This is obtained from the gimbal event object.
 @param fenceId The Id of the geofence - This is obtained from the gimbal event object.
 @param dwellTime An optional string containing the dwell time inside the geofence (determined on exit).
 @param optionalData An optional dictionary containing additional query parameters.
 @return KMRequest
 @see reportAppEvent:withData:
 @warning Deprecated.  Use reportAppEvent:withData:
 */

-(KMRequest*) reportGeofenceEvent: (NSString*) fenceEvent FenceName: (NSString*) fenceName FenceId: (NSString*) fenceId OptionalDwellTime: (NSString*) dwellTime withOptionalData: (NSMutableDictionary*) optionalData __deprecated;

/** Reports an App Event of a specific type with a dictionary of name/value pairs as the payload.
 
 Event Type: 
 
 - appopen
 - appclose

 Request:
 
     {
     }
 
 
 
 Event Type:
 
 - influencedopen
 
 Request:
 
     {
         “source”: String,
         "payload”: String or Object
     }
 
 - source
 
 A string indicating the triggering source of the open (e.g. “push”, "web", “app”).
 
 - payload
 
 The payload associated with the triggered app open.
 
 
 
 Event Type:
 
 - pushreg
 
 Request:
 
     {
         “platform”: Enum, //ios, and
         "device_id”: String
     }
 
 - platform
 
 One of the following device platforms: “ios” or “and”.
 
 - device_id
 
 The identifier (i.e. push token) obtained from the platform’s push service for the registering consumer device.
 
 
 
 Event Type:
 
 - fencein
 - fenceout
 
 Request: 
 
     {
         "fenceId”: Number,
         "fenceName”: String,
         "dwellTime”: Number //(seconds) Only for fenceout event
     }
 
 - fenceId
 
 Id of defined Geofence
 
 - fenceName
 
 Name of defined Geofence
 
 - dwellTime
 
 The amount of time, in seconds, that the Consumer spent within the defined Geofence.
 
 
 
 Event Type:
 
 - locupdate
 
 Request:
 
     {
         locHistory: [
             {  
                 isotime:  String,
                 lat:  String, // floating point latitude value
                 lon:  String  // floating point longitude value
             },  ...
         ]
     }
 
 - isotime
 
 Time that call is made or the time of a relevant event, in ISO 8601 format (YYYY-MM-DDThh:mm:ss.sss{Z | [+|-]hh:mm}).  Optional.
 
 - lat
 - lon

 Latitude and longitude of current location at the time of the call or the relevant event, in decimal format (e.g. -96.820317).  Optional.
 
 Response:
 
     {
         “code”: Number,
         “message”: String,
         “version”: String,
         "sdkConfig" : {
             "locCapture" : {
                 "enable" : Boolean,
                 "bufferSize" : Number //The number of historical location points to store locally
             }
         }
     }
     
 @param eventType The type of event to process
 @param eventData A dictionary containing the data to be sent along with the event
 @return KMRequest
 */

-(KMRequest*) reportAppEvent: (NSString*) eventType withData: (NSMutableDictionary*) eventData;

@end
